@@ -16,7 +16,9 @@ import ConfigParser
 import types
 import logging
 import shutil
+import fnmatch
 from string import Template
+
 logging.basicConfig(level=logging.DEBUG)
 
 try:
@@ -27,8 +29,10 @@ except:
 sys.path.append(os.path.dirname(os.path.abspath(__file__)) + "/../../ext/threadpool-1.2.7/src/")
 logging.debug(str(sys.path))
 from threadpool import *
-from bigjob import logger
 
+# BigJob/Pilot framework classes
+from bigjob import logger
+from pilot.impl.pilotdata_manager import PilotData, PilotDataService
 
 logger.debug("Python Version: " + str(sys.version_info))
 if sys.version_info < (2, 5):
@@ -148,8 +152,11 @@ class bigjob_agent:
                 logger.error("ZMQ Backend not found. Please install ZeroMQ (http://www.zeromq.org/intro:get-the-software) and " 
                       +"PYZMQ (http://zeromq.github.com/pyzmq/)")
 
+        ###
+        # Initiate coordination sub-system of both BJ agent and Pilot Data
         self.coordination = bigjob_coordination(server_connect_url=self.coordination_url)
-    
+        self.pilot_data_service = PilotDataService(coordination_url=self.coordination_url)
+        
         # update state of pilot job to running
         logger.debug("set state to : " +  str(bigjob.state.Running))
         self.coordination.set_pilot_state(self.base_url, str(bigjob.state.Running), False)
@@ -328,7 +335,6 @@ class bigjob_agent:
                     logger.debug("Directory %s already exists."%workingdirectory)
                 logging.debug("Sub-Job: %s, Working_directory: %s"%(job_id, workingdirectory))
                 
-                
                 output="stdout"
                 if (job_dict.has_key("Output") == True):
                     output = job_dict["Output"]
@@ -341,9 +347,15 @@ class bigjob_agent:
                 if not os.path.isabs(error):
                     error=os.path.join(workingdirectory, error)
                 
+                                
                 # append job to job list
                 self.jobs.append(job_url)
                 
+                
+                # File Stage-In of dependent data units
+                if job_dict.has_key("InputData"):
+                    self.__stage_in_data_units(eval(job_dict["InputData"]), workingdirectory)
+                
                 # File Stage-In - Move pilot-level files to working directory of sub-job
                 if self.pilot_description!=None:
                     try:
@@ -440,9 +452,9 @@ class bigjob_agent:
                 self.coordination.set_job_state(job_url, str(bigjob.state.Running))
             except:
                 traceback.print_exc(file=sys.stderr)
+                
     
-   
-            
+             
     def allocate_nodes(self, job_dict):
         """ allocate nodes
             allocated nodes will be written to machinefile advert-launcher-machines-<jobid>
@@ -622,8 +634,20 @@ class bigjob_agent:
                 p_state = p.poll()
                 logger.debug(self.print_job(i) + " state: " + str(p_state) + " return code: " + str(p.returncode))
                 if (p_state != None and (p_state==0 or p_state==255)):
-                    logger.debug("Job successful: " + self.print_job(i) + " - set state to Done")
-                    self.update_output_file()
+                    logger.debug("** Job successful: " + self.print_job(i) + " - set state to Done **")
+                    ###########################################################
+                    # Handle stage-out
+                    self.update_output_file() # for Condor case
+                    job_dict = self.coordination.get_job(i) # for Pilot Data case
+                    if job_dict.has_key("OutputData"):
+                        workingdirectory = os.path.join(os.getcwd(), job_dict["job-id"])  
+                        if (job_dict.has_key("WorkingDirectory") == True):
+                            workingdirectory =  job_dict["WorkingDirectory"]
+                            workingdirectory = self.__expand_directory(workingdirectory)
+                        self.__stage_out_data_units(eval(job_dict["OutputData"]), workingdirectory)
+                    
+                    ###########################################################
+                    # Status update
                     self.coordination.set_job_state(i, str(bigjob.state.Done))
                     self.free_nodes(i)
                     del self.processes[i]
@@ -701,6 +725,68 @@ class bigjob_agent:
         self.stop=True
         
     
+    #############################################################################
+    # Private methods
+    
+    def __stage_in_data_units(self, input_data=[], target_directory="."):
+        """ stage in data units specified in input_data field """
+        logger.debug("Stage in input files to: %s"%target_directory)
+        for i in input_data:
+            pd_url = self.__get_pd_url(i)
+            du_id = self.__get_du_id(i)
+            pd = PilotData(pd_url=pd_url)
+            du = pd.get_du(du_id)
+            du.export(target_directory)
+    
+    
+    def __stage_out_data_units(self, output_data=[], workingdirectory=None):
+        """ stage out data to a specified data unit pilot data """
+        logger.debug("Stage out output files")
+        
+        """ Parsing output data field of job description:
+            {
+            ...
+             "output_data": [
+                            {
+                             output_data_unit.get_url(): 
+                             ["stdout.txt", "stderr.txt"]
+                            }
+                            ]
+            }    
+        """
+        for data_unit_dict in output_data: 
+            logger.debug("Process: " + str(data_unit_dict))
+            for du_url in data_unit_dict.keys(): # go through all dicts (each representing 1 PD) 
+                pd_url = self.__get_pd_url(du_url)
+                du_id = self.__get_du_id(du_url)
+                pilot_data = PilotData(pd_url=pd_url)
+                du = pilot_data.get_du(du_id)
+                file_list = data_unit_dict[du_url]
+                logger.debug("Add files: " + str(file_list))
+                all_files=[]
+                for output_file in file_list:
+                    expanded_files = [output_file]
+                    if output_file.find("*")>=0 or output_file.find("?")>=0:
+                        expanded_files = self.__expand_file_pattern(output_file, workingdirectory)
+                        logger.debug("Expanded files: " + str(expanded_files))
+                        
+                    for f in expanded_files:
+                        all_files.append(os.path.join(workingdirectory, f))
+                 
+                du.add_files(all_files)                        
+                for f in all_files:       
+                    os.remove(f)
+    
+    def __expand_file_pattern(self, filename_pattern, workingdirectory):
+        """ expand files with wildcard * to a list """
+        files = os.listdir(workingdirectory)
+        logger.debug("All files in directory: " + str(files))
+        matches = []
+        for i in files:
+            if fnmatch.fnmatch(i, filename_pattern):
+                matches.append(i)
+        return matches
+    
     def __expand_directory(self, directory):
         """ expands directory name $HOME or ~ to the working directory
             on the respective machine 
@@ -717,8 +803,15 @@ class bigjob_agent:
             pass
         
         return directory
+    
 
-            
+    def __get_pd_url(self, du_url):
+        url = du_url[:du_url.index(":du-")]
+        return url
+
+    def __get_du_id(self, du_url):
+        du_id = du_url[du_url.index("du-"):]
+        return du_id        
     
     def __get_launch_method(self, requested_method):
         """ returns desired execution method: ssh, aprun """
